==> 4.5.0.tar.gz: terraform-provider-decort/internal/service/cloudbroker/lb/resource_lb_frontend_bind.go <==
/*
Copyright (c) 2019-2023 Digital Energy Cloud Solutions LLC. All Rights Reserved.
Authors:
Petr Krutov, <petr.krutov@digitalenergy.online>
Stanislav Solovev, <spsolovev@digitalenergy.online>
Kasim Baybikov, <kmbaybikov@basistech.ru>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/*
Terraform DECORT provider - manage resources provided by DECORT (Digital Energy Cloud
Orchestration Technology) with Terraform by Hashicorp.

Source code: https://repository.basistech.ru/BASIS/terraform-provider-decort

Please see README.md to learn where to place source code so that it
builds seamlessly.

Documentation: https://repository.basistech.ru/BASIS/terraform-provider-decort/wiki
*/

package lb

import (
	"context"
	"strconv"
	"strings"

	"github.com/hashicorp/terraform-plugin-sdk/v2/diag"
	"github.com/hashicorp/terraform-plugin-sdk/v2/helper/schema"
	log "github.com/sirupsen/logrus"
	"repository.basistech.ru/BASIS/decort-golang-sdk/pkg/cloudbroker/lb"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/constants"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/controller"
)

func resourceLBFrontendBindCreate(ctx context.Context, d *schema.ResourceData, m interface{}) diag.Diagnostics {
	log.Debugf("resourceLBFrontendBindCreate")

	haveLBID, err := existLBID(ctx, d, m)
	if err != nil {
		return diag.FromErr(err)
	}

	if !haveLBID {
		return diag.Errorf("resourceLBFrontendBindCreate: can't create LB frontend bind because LBID %d is not allowed or does not exist", d.Get("lb_id").(int))
	}

	c := m.(*controller.ControllerCfg)
	req := lb.FrontendBindRequest{
		LBID:           uint64(d.Get("lb_id").(int)),
		FrontendName:   d.Get("frontend_name").(string),
		BindingName:    d.Get("name").(string),
		BindingAddress: d.Get("address").(string),
		BindingPort:    uint64(d.Get("port").(int)),
	}

	_, err = c.CloudBroker().LB().FrontendBind(ctx, req)
	if err != nil {
		return diag.FromErr(err)
	}

	d.SetId(strconv.Itoa(d.Get("lb_id").(int)) + "#" + d.Get("frontend_name").(string) + "#" + d.Get("name").(string))

	_, err = utilityLBFrontendBindCheckPresence(ctx, d, m)
	if err != nil {
		return diag.FromErr(err)
	}

	return resourceLBFrontendBindRead(ctx, d, m)
}

func resourceLBFrontendBindRead(ctx context.Context, d *schema.ResourceData, m interface{}) diag.Diagnostics {
	log.Debugf("resourceLBFrontendBindRead")

	b, err := utilityLBFrontendBindCheckPresence(ctx, d, m)
	if err != nil {
		d.SetId("")
		return diag.FromErr(err)
	}

	lbId, _ := strconv.ParseInt(strings.Split(d.Id(), "#")[0], 10, 32)
	frontendName := strings.Split(d.Id(), "#")[1]

	flattenLBFrontendBind(d, b, lbId, frontendName)

	return nil
}

func resourceLBFrontendBindDelete(ctx context.Context, d *schema.ResourceData, m interface{}) diag.Diagnostics {
	log.Debugf("resourceLBFrontendBindDelete")

	_, err := utilityLBFrontendBindCheckPresence(ctx, d, m)
	if err != nil {
		return diag.FromErr(err)
	}

	c := m.(*controller.ControllerCfg)
	req := lb.FrontendBindDeleteRequest{
		LBID:         uint64(d.Get("lb_id").(int)),
		FrontendName: d.Get("frontend_name").(string),
		BindingName:  d.Get("name").(string),
	}

	_, err = c.CloudBroker().LB().FrontendBindDelete(ctx, req)
	if err != nil {
		return diag.FromErr(err)
	}

	d.SetId("")

	return nil
}

func resourceLBFrontendBindUpdate(ctx context.Context, d *schema.ResourceData, m interface{}) diag.Diagnostics {
	log.Debugf("resourceLBFrontendBindEdit")
	c := m.(*controller.ControllerCfg)

	haveLBID, err := existLBID(ctx, d, m)
	if err != nil {
		return diag.FromErr(err)
	}

	if !haveLBID {
		return diag.Errorf("resourceLBFrontendBindUpdate: can't update LB frontend bind because LBID %d is not allowed or does not exist", d.Get("lb_id").(int))
	}

	req := lb.FrontendBindUpdateRequest{
		FrontendName: d.Get("frontend_name").(string),
		BindingName:  d.Get("name").(string),
		LBID:         uint64(d.Get("lb_id").(int)),
	}

	if d.HasChange("address") ||  d.HasChange("port") {
		req.BindingAddress = d.Get("address").(string)
		req.BindingPort = uint64(d.Get("port").(int))
	}

	_, err = c.CloudBroker().LB().FrontendBindUpdate(ctx, req)
	if err != nil {
		return diag.FromErr(err)
	}

	return resourceLBFrontendBindRead(ctx, d, m)
}

func ResourceLBFrontendBind() *schema.Resource {
	return &schema.Resource{
		SchemaVersion: 1,

		CreateContext: resourceLBFrontendBindCreate,
		ReadContext:   resourceLBFrontendBindRead,
		UpdateContext: resourceLBFrontendBindUpdate,
		DeleteContext: resourceLBFrontendBindDelete,

		Importer: &schema.ResourceImporter{
			StateContext: schema.ImportStatePassthroughContext,
		},

		Timeouts: &schema.ResourceTimeout{
			Create:  &constants.Timeout600s,
			Read:    &constants.Timeout300s,
			Update:  &constants.Timeout300s,
			Delete:  &constants.Timeout300s,
			Default: &constants.Timeout300s,
		},

		Schema: map[string]*schema.Schema{
			"lb_id": {
				Type:        schema.TypeInt,
				Required:    true,
				Description: "ID of the LB instance to backendCreate",
			},
			"frontend_name": {
				Type:        schema.TypeString,
				Required:    true,
				Description: "Must be unique among all backends of this LB - name of the new backend to create",
			},
			"address": {
				Type:     schema.TypeString,
				Required: true,
			},
			"guid": {
				Type:     schema.TypeString,
				Computed: true,
			},
			"name": {
				Type:     schema.TypeString,
				Required: true,
			},
			"port": {
				Type:     schema.TypeInt,
				Required: true,
			},
		},
	}
}

==> 4.7.4.tar.gz: terraform-provider-decort/internal/service/cloudbroker/lb/resource_lb_frontend_bind.go <==
/*
Copyright (c) 2019-2023 Digital Energy Cloud Solutions LLC. All Rights Reserved.
Authors:
Petr Krutov, <petr.krutov@digitalenergy.online>
Stanislav Solovev, <spsolovev@digitalenergy.online>
Kasim Baybikov, <kmbaybikov@basistech.ru>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/*
Terraform DECORT provider - manage resources provided by DECORT (Digital Energy Cloud
Orchestration Technology) with Terraform by Hashicorp.

Source code: https://repository.basistech.ru/BASIS/terraform-provider-decort

Please see README.md to learn where to place source code so that it
builds seamlessly.

Documentation: https://repository.basistech.ru/BASIS/terraform-provider-decort/wiki
*/

package lb

import (
	"context"
	"strconv"
	"strings"

	"github.com/hashicorp/terraform-plugin-sdk/v2/diag"
	"github.com/hashicorp/terraform-plugin-sdk/v2/helper/schema"
	log "github.com/sirupsen/logrus"
	"repository.basistech.ru/BASIS/decort-golang-sdk/pkg/cloudbroker/lb"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/constants"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/controller"
)

func resourceLBFrontendBindCreate(ctx context.Context, d *schema.ResourceData, m interface{}) diag.Diagnostics {
	log.Debugf("resourceLBFrontendBindCreate: call for lb_id %d, frontend %s to create bind %s",
		d.Get("lb_id").(int),
		d.Get("frontend_name").(string),
		d.Get("name").(string))

	c := m.(*controller.ControllerCfg)

	if diags := checkParamsExistenceLb(ctx, d, c); diags != nil {
		return diags
	}

	req := lb.FrontendBindRequest{
		LBID:           uint64(d.Get("lb_id").(int)),
		FrontendName:   d.Get("frontend_name").(string),
		BindingName:    d.Get("name").(string),
		BindingAddress: d.Get("address").(string),
		BindingPort:    uint64(d.Get("port").(int)),
	}

	_, err := c.CloudBroker().LB().FrontendBind(ctx, req)
	if err != nil {
		return diag.FromErr(err)
	}

	d.SetId(strconv.Itoa(d.Get("lb_id").(int)) + "#" + d.Get("frontend_name").(string) + "#" + d.Get("name").(string))

	return resourceLBFrontendBindRead(ctx, d, m)
}

func resourceLBFrontendBindRead(ctx context.Context, d *schema.ResourceData, m interface{}) diag.Diagnostics {
	log.Debugf("resourceLBFrontendBindRead: call for %s", d.Id())

	b, err := utilityLBFrontendBindCheckPresence(ctx, d, m)
	if err != nil {
		d.SetId("")
		return diag.FromErr(err)
	}

	lbId, _ := strconv.ParseInt(strings.Split(d.Id(), "#")[0], 10, 32)
	frontendName := strings.Split(d.Id(), "#")[1]

	flattenLBFrontendBind(d, b, lbId, frontendName)

	return nil
}

func resourceLBFrontendBindDelete(ctx context.Context, d *schema.ResourceData, m interface{}) diag.Diagnostics {
	log.Debugf("resourceLBFrontendBindDelete: call for %s", d.Id())

	_, err := utilityLBFrontendBindCheckPresence(ctx, d, m)
	if err != nil {
		d.SetId("")
		return diag.FromErr(err)
	}

	c := m.(*controller.ControllerCfg)
	req := lb.FrontendBindDeleteRequest{
		LBID:         uint64(d.Get("lb_id").(int)),
		FrontendName: d.Get("frontend_name").(string),
		BindingName:  d.Get("name").(string),
	}

	_, err = c.CloudBroker().LB().FrontendBindDelete(ctx, req)
	if err != nil {
		return diag.FromErr(err)
	}

	d.SetId("")

	return nil
}

func resourceLBFrontendBindUpdate(ctx context.Context, d *schema.ResourceData, m interface{}) diag.Diagnostics {
	log.Debugf("resourceLBFrontendBindEdit: call for %s", d.Id())
	c := m.(*controller.ControllerCfg)

	if diags := checkParamsExistenceLb(ctx, d, c); diags != nil {
		return diags
	}

	req := lb.FrontendBindUpdateRequest{
		FrontendName: d.Get("frontend_name").(string),
		BindingName:  d.Get("name").(string),
		LBID:         uint64(d.Get("lb_id").(int)),
	}

	if d.HasChange("address") || d.HasChange("port") {
		req.BindingAddress = d.Get("address").(string)
		req.BindingPort = uint64(d.Get("port").(int))
	}

	_, err := c.CloudBroker().LB().FrontendBindUpdate(ctx, req)
	if err != nil {
		return diag.FromErr(err)
	}

	return resourceLBFrontendBindRead(ctx, d, m)
}

func ResourceLBFrontendBind() *schema.Resource {
	return &schema.Resource{
		SchemaVersion: 1,

		CreateContext: resourceLBFrontendBindCreate,
		ReadContext:   resourceLBFrontendBindRead,
		UpdateContext: resourceLBFrontendBindUpdate,
		DeleteContext: resourceLBFrontendBindDelete,

		Importer: &schema.ResourceImporter{
			StateContext: schema.ImportStatePassthroughContext,
		},

		Timeouts: &schema.ResourceTimeout{
			Create:  &constants.Timeout600s,
			Read:    &constants.Timeout300s,
			Update:  &constants.Timeout300s,
			Delete:  &constants.Timeout300s,
			Default: &constants.Timeout300s,
		},

		Schema: resourceLbFrontendBindSchemaMake(),
	}
}
